@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 - 2025 grunfink et al. / MIT license */
 
-#define VERSION "2.72"
+#define VERSION "2.75"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -16,6 +16,10 @@
 #define MAX_THREADS 256
 #endif
 
+#ifndef MAX_JSON_DEPTH
+#define MAX_JSON_DEPTH 8
+#endif
+
 #ifndef MAX_CONVERSATION_LEVELS
 #define MAX_CONVERSATION_LEVELS 48
 #endif
@@ -29,10 +33,11 @@ extern xs_str *srv_basedir;
 extern xs_dict *srv_config;
 extern xs_str *srv_baseurl;
 extern xs_str *srv_proxy_token_seed;
+extern xs_dict *srv_langs;
 
 extern int dbglevel;
 
-#define L(s) (s)
+#define L(s) lang_str((s), user)
 
 #define POSTLIKE_OBJECT_TYPE "Note|Question|Page|Article|Video|Audio|Event"
 
@@ -55,6 +60,8 @@ typedef struct {
     xs_dict *links;     /* validated links */
     xs_str *actor;      /* actor url */
     xs_str *md5;        /* actor url md5 */
+    const xs_dict *lang;/* string translation dict */
+    const char *tz;     /* configured timezone */
 } snac;
 
 typedef struct {
@@ -199,6 +206,12 @@ void draft_del(snac *user, const char *id);
 void draft_add(snac *user, const char *id, const xs_dict *msg);
 xs_list *draft_list(snac *user);
 
+int is_scheduled(snac *user, const char *id);
+void schedule_del(snac *user, const char *id);
+void schedule_add(snac *user, const char *id, const xs_dict *msg);
+xs_list *scheduled_list(snac *user);
+void scheduled_process(snac *user);
+
 int limited(snac *user, const char *id, int cmd);
 #define is_limited(user, id) limited((user), (id), 0)
 #define limit(user, id) limited((user), (id), 1)
@@ -323,7 +336,7 @@ xs_dict *msg_follow(snac *snac, const char *actor);
 
 xs_dict *msg_note(snac *snac, const xs_str *content, const xs_val *rcpts,
                   const xs_str *in_reply_to, const xs_list *attach,
-                  int scope, const char *lang);
+                  int scope, const char *lang_str, const char *msg_date);
 
 xs_dict *msg_undo(snac *snac, const xs_val *object);
 xs_dict *msg_delete(snac *snac, const char *id);
@@ -440,3 +453,5 @@ xs_str *make_url(const char *href, const char *proxy, int by_token);
 
 int badlogin_check(const char *user, const char *addr);
 void badlogin_inc(const char *user, const char *addr);
+
+const char *lang_str(const char *str, const snac *user);
