@@ -1,11 +1,11 @@
-/* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
+/* copyright (c) 2022 - 2025 grunfink et al. / MIT license */
 
 #ifndef _XS_HTTPD_H
 
 #define _XS_HTTPD_H
 
 xs_dict *xs_httpd_request(FILE *f, xs_str **payload, int *p_size);
-void xs_httpd_response(FILE *f, int status, xs_dict *headers, xs_str *body, int b_size);
+void xs_httpd_response(FILE *f, int status, const char *status_text, xs_dict *headers, xs_str *body, int b_size);
 
 
 #ifdef XS_IMPLEMENTATION
@@ -15,40 +15,48 @@ xs_dict *xs_httpd_request(FILE *f, xs_str **payload, int *p_size)
 {
     xs *q_vars = NULL;
     xs *p_vars = NULL;
-    xs *l1, *l2;
-    char *v;
+    xs *l1;
+    const char *v;
+    char *saveptr;
 
     xs_socket_timeout(fileno(f), 2.0, 0.0);
 
     /* read the first line and split it */
     l1 = xs_strip_i(xs_readline(f));
-    l2 = xs_split(l1, " ");
+    char *raw_path;
+    const char *mtd;
+    const char *proto;
 
-    if (xs_list_len(l2) != 3) {
-        /* error or timeout */
+    if (!(mtd = strtok_r(l1, " ", &saveptr)) ||
+        !(raw_path = strtok_r(NULL, " ", &saveptr)) ||
+        !(proto = strtok_r(NULL, " ", &saveptr)) ||
+        strtok_r(NULL, " ", &saveptr))
+        return NULL;
+
+    if (!xs_is_string(mtd) || !xs_is_string(raw_path) || !xs_is_string(proto))
         return NULL;
-    }
 
     xs_dict *req = xs_dict_new();
 
-    req = xs_dict_append(req, "method", xs_list_get(l2, 0));
-    req = xs_dict_append(req, "proto",  xs_list_get(l2, 2));
+    req = xs_dict_append(req, "method", mtd);
+    req = xs_dict_append(req, "raw_path", raw_path);
+    req = xs_dict_append(req, "proto",  proto);
 
     {
-        /* split the path with its optional variables */
-        xs *udp = xs_url_dec(xs_list_get(l2, 1));
-        xs *pnv = xs_split_n(udp, "?", 1);
-
-        /* store the path */
-        req = xs_dict_append(req, "path", xs_list_get(pnv, 0));
+        char *q = strchr(raw_path, '?');
 
         /* get the variables */
-        q_vars = xs_url_vars(xs_list_get(pnv, 1));
+        if (q) {
+                *q++ = '\0';
+                q_vars = xs_url_vars(q);
+        }
+        /* store the path */
+        req = xs_dict_append(req, "path", raw_path);
     }
 
     /* read the headers */
     for (;;) {
-        xs *l, *p = NULL;
+        xs *l;
 
         l = xs_strip_i(xs_readline(f));
 
@@ -57,10 +65,18 @@ xs_dict *xs_httpd_request(FILE *f, xs_str **payload, int *p_size)
             break;
 
         /* split header and content */
-        p = xs_split_n(l, ": ", 1);
+        char *cnt = strchr(l, ':');
+        if (!cnt)
+            continue;
 
-        if (xs_list_len(p) == 2)
-            req = xs_dict_append(req, xs_tolower_i(xs_list_get(p, 0)), xs_list_get(p, 1));
+        *cnt++ = '\0';
+        cnt += strspn(cnt, " \r\n\t\v\f");
+        l = xs_rstrip_chars_i(l, " \r\n\t\v\f");
+
+        if (!xs_is_string(cnt))
+            continue;
+
+        req = xs_dict_append(req, xs_tolower_i(l), cnt);
     }
 
     xs_socket_timeout(fileno(f), 5.0, 0.0);
@@ -74,8 +90,7 @@ xs_dict *xs_httpd_request(FILE *f, xs_str **payload, int *p_size)
     v = xs_dict_get(req, "content-type");
 
     if (*payload && v && strcmp(v, "application/x-www-form-urlencoded") == 0) {
-        xs *upl = xs_url_dec(*payload);
-        p_vars  = xs_url_vars(upl);
+        p_vars  = xs_url_vars(*payload);
     }
     else
     if (*payload && v && xs_startswith(v, "multipart/form-data")) {
@@ -94,18 +109,17 @@ xs_dict *xs_httpd_request(FILE *f, xs_str **payload, int *p_size)
 }
 
 
-void xs_httpd_response(FILE *f, int status, xs_dict *headers, xs_str *body, int b_size)
+void xs_httpd_response(FILE *f, int status, const char *status_text, xs_dict *headers, xs_str *body, int b_size)
 /* sends an httpd response */
 {
     xs *proto;
-    xs_str *k;
-    xs_val *v;
+    const xs_str *k;
+    const xs_val *v;
 
-    proto = xs_fmt("HTTP/1.1 %d %s", status, status / 100 == 2 ? "OK" : "ERROR");
+    proto = xs_fmt("HTTP/1.1 %d %s", status, status_text);
     fprintf(f, "%s\r\n", proto);
 
-    int c = 0;
-    while (xs_dict_next(headers, &k, &v, &c)) {
+    xs_dict_foreach(headers, k, v) {
         fprintf(f, "%s: %s\r\n", k, v);
     }
 
